import re
from nonebot import on_command
from nonebot.adapters.onebot.v11 import (
    PRIVATE_FRIEND,
    Bot,
    Event,
    GROUP,
    GROUP_ADMIN,
    GROUP_OWNER,
    Message,
    MessageEvent,
    GroupMessageEvent,
    MessageSegment,
)

from ..xiuxian2_handle import XiuxianDateManage, OtherSet
from ..data_source import jsondata
from .draw_user_info import draw_user_info_img
from ..cd_manager import add_cd, check_cd, cd_msg
from .infoconfig import get_config
from ..utils import data_check_conf

config = get_config()




xiuxian_message = on_command("我的修仙信息", aliases={"我的存档"}, priority=5)


sql_message = XiuxianDateManage()  # sql类

@xiuxian_message.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    """我的修仙信息"""
    await data_check_conf(bot, event)

    if cd := check_cd(event, '查询信息'):
        # 如果 CD 还没到 则直接结束
        await xiuxian_message.finish(cd_msg(cd), at_sender=True)
        
    
    try:
        user_id, group_id, mess = await data_check(bot, event)
    except MsgError:
        return
    add_cd(event, config['查询CD'], '查询信息')
    mess = sql_message.get_user_real_info(user_id)
    user_name = mess.user_name
    if user_name:
        pass
    else:
        user_name = "无名氏(发送改名+道号更新)"
    level_rate = sql_message.get_root_rate(mess.root_type)  # 灵根倍率
    realm_rate = jsondata.level_data()[mess.level]["spend"]  # 境界倍率
    sect_id = mess.sect_id
    if sect_id:
        sect_info = sql_message.get_sect_info(sect_id)
        sectmsg = sect_info.sect_name
        sectzw = jsondata.sect_config_data()[f"{mess.sect_position}"]["title"]
    else:
        sectmsg = '无宗门'
        sectzw = '无'

    # 判断突破的修为
    list_all = len(OtherSet().level) - 1
    now_index = OtherSet().level.index(mess.level)
    if list_all == now_index:
        exp_meg = "位面至高"
    else:
        is_updata_level = OtherSet().level[now_index + 1]
        need_exp = sql_message.get_level_power(is_updata_level)
        get_exp = need_exp - mess.exp
        if get_exp > 0:
            if get_exp > 10000:
                get_exp = int(divmod(get_exp, 10000)[0])
                exp_meg = "还需{}万修为可突破".format(get_exp)
            else:
                exp_meg = "还需{}修为可突破".format(get_exp)
        else:
            exp_meg = "可突破！"
    
    DETAIL_MAP = {
    '道号': f'{user_name}',
    '境界': f'{mess.level}',
    '修为': f'{mess.exp}',
    '灵石': f'{mess.stone}',
    '战力': f'{int(mess.exp * level_rate * realm_rate)}',
    '灵根': f'{mess.root}({mess.root_type}+{int(level_rate * 100)}%)',
    '突破状态': f'{exp_meg}，概率：{jsondata.level_rate_data()[mess.level] + int(mess.level_up_rate)}%',
    '攻击力': f'{mess.atk}，攻修等级{mess.atkpractice}级',
    '所在宗门':sectmsg,
    '宗门职位':sectzw,
    }
    if config['是否开启图片信息']:
        img_res = await draw_user_info_img(user_id, DETAIL_MAP)
        await xiuxian_message.finish(MessageSegment.image(img_res), at_sender=True)
    else:
        msg = f"""{user_name}道友的信息
灵根为：{mess.root}({mess.root_type}+{int(level_rate * 100)}%)
当前境界：{mess.level}(境界+{int(realm_rate * 100)}%)
当前灵石：{mess.stone}
当前修为：{mess.exp}(修炼效率+{int((level_rate * realm_rate) * 100)}%)
突破状态：{exp_meg}
你的战力为：{int(mess.exp * level_rate * realm_rate)}"""
        await xiuxian_message.finish(msg, at_sender=True)
    

async def data_check(bot, event):
    user_qq = event.get_user_id()
    group_id = await get_group_id(event.get_session_id())
    msg = sql_message.get_user_message(user_qq)

    if msg:
        pass
    else:
        await bot.send(event=event, message=f"没有您的信息，输入【我要修仙】加入！")
        raise MsgError

    return user_qq, group_id, msg

async def get_group_id(session_id):
    """获取group_id"""
    res = re.findall("_(.*)_", session_id)
    group_id = res[0]
    return group_id

class MsgError(ValueError):
    pass







import json
import os
from pathlib import Path

configkey = ["查询CD", "是否开启图片信息"]
CONFIG = {
    "查询CD" :60,
    "是否开启图片信息":True,
}

def get_config():
    try:
        config = readf()
        for key in configkey:
            if key not in list(config.keys()):
                config[key] = CONFIG[key]
        savef(config)
    except:
        config = CONFIG
        savef(config)
    return config

CONFIGJSONPATH = Path(__file__).parent
FILEPATH = CONFIGJSONPATH / 'config.json'
def readf():
    with open(FILEPATH, "r", encoding="UTF-8") as f:
        data = f.read()
    return json.loads(data)


def savef(data):
    data = json.dumps(data, ensure_ascii=False, indent=3)
    savemode = "w" if os.path.exists(FILEPATH) else "x"
    with open(FILEPATH, mode=savemode, encoding="UTF-8") as f:
        f.write(data)
        f.close
    return True


import httpx
from nonebot.log import logger
import asyncio
import hashlib
import os
from PIL import Image
import io
from pathlib import Path

async def download_url(url: str) -> bytes:
    async with httpx.AsyncClient() as client:
        for i in range(3):
            try:
                resp = await client.get(url, timeout=20)
                resp.raise_for_status()
                return resp.content
            except Exception as e:
                logger.warning(f"Error downloading {url}, retry {i}/3: {e}")
                await asyncio.sleep(3)
    raise Exception(f"{url} 下载失败！")

async def download_avatar(user_id: str) -> bytes:
    url = f"http://q1.qlogo.cn/g?b=qq&nk={user_id}&s=640"
    data = await download_url(url)
    if hashlib.md5(data).hexdigest() == "acef72340ac0e914090bd35799f5594e":
        url = f"http://q1.qlogo.cn/g?b=qq&nk={user_id}&s=100"
        data = await download_url(url)
    return data

async def get_avatar_by_user_id_and_save(user_id):
    user_id = str(user_id)
    PLAYERSDATA = Path() / "data" / "xiuxian" / "players"
    
    USER_AVATAR_PATH = PLAYERSDATA / user_id / 'AVATAR.png'
    
    if os.path.exists(USER_AVATAR_PATH):
        logger.info("头像已存在")
        im = Image.open(USER_AVATAR_PATH).resize((280, 280)).convert("RGBA")
        return im
    else:
        try:
            logger.info("头像不存在，开始下载")
            image_bytes = await download_avatar(user_id)
            im = Image.open(io.BytesIO(image_bytes)).resize((280, 280)).convert("RGBA")
            im.save(USER_AVATAR_PATH, "PNG")
        except:
            logger.error("获取头像出错")
    
        return im


from PIL import Image, ImageDraw, ImageFont
import asyncio
import asyncio
from pathlib import Path

from PIL import Image
from .download import get_avatar_by_user_id_and_save

from .send_image_tool import convert_img


TEXT_PATH = Path(__file__).parent / 'texture2d'

first_color = (242, 250, 242)
second_color = (57, 57, 57)


FONT_ORIGIN_PATH = Path(__file__).parent / 'font.ttf'

def font_origin(size: int) -> ImageFont.FreeTypeFont:
    return ImageFont.truetype(str(FONT_ORIGIN_PATH), size=size)


font_36 = font_origin(36)
font_40 = font_origin(40)
font_24 = font_origin(24)


async def draw_user_info_img(user_id, DETAIL_MAP):
    based_w = 1100
    based_h = 2250
    #获取背景图
    img = Image.open(TEXT_PATH / 'back.png').resize((based_w, based_h)).convert("RGBA")
    #获取用户头像圆框
    user_status = Image.open(TEXT_PATH / 'user_state.png').resize((450, 450)).convert("RGBA")
    temp = await get_avatar_by_user_id_and_save(user_id)
    user_avatar = await img_author(temp, user_status)
    r, g, b, a = user_status.split()
    #绘制头像框位置
    img.paste(user_avatar, (100, 100), mask=a) 
    # img_draw = ImageDraw.Draw(img)
    #h获取信息图片
    line = Image.open(TEXT_PATH / 'line3.png').resize((400, 60)).convert("RGBA")
    line_draw = ImageDraw.Draw(line)
    word = f"QQ:{user_id}"
    w, h = await linewh(line, word)
    line_draw.text((w, h), word, first_color, font_36, 'lm')
    #绘制QQ信息
    img.paste(line, (130,520), line)
    
    DETAIL_baseinfo = {}
    DETAIL_baseinfo["灵根"] = DETAIL_MAP["灵根"]
    DETAIL_baseinfo["突破状态"] = DETAIL_MAP["突破状态"]
    DETAIL_baseinfo["攻击力"] = DETAIL_MAP["攻击力"]
    
    DETAIL_right = {}
    DETAIL_right['道号'] = DETAIL_MAP['道号']
    DETAIL_right['境界'] = DETAIL_MAP['境界']
    DETAIL_right['修为'] = DETAIL_MAP['修为']
    DETAIL_right['灵石'] = DETAIL_MAP['灵石']
    DETAIL_right['战力'] = DETAIL_MAP['战力']
    
    tasks1 = []
    for key, value in DETAIL_right.items():
        tasks1.append(_draw_line(img, key, value, DETAIL_right))
    await asyncio.gather(*tasks1)
    
    baseinfo = Image.open(TEXT_PATH / 'line2.png').resize((900, 100)).convert("RGBA")
    baseword = '【基本信息】'
    w, h = await linewh(baseinfo, baseword)
    baseinfo_draw = ImageDraw.Draw(baseinfo)
    baseinfo_draw.text((w, h), baseword, first_color, font_40, 'lm')
    img.paste(baseinfo, (100, 600), baseinfo)
    
    tasks2 = []
    for key, value in DETAIL_baseinfo.items():
        tasks2.append(_draw_base_info_line(img, key, value, DETAIL_baseinfo))
    await asyncio.gather(*tasks2)
    
    sectinfo = Image.open(TEXT_PATH / 'line2.png').resize((900, 100)).convert("RGBA")
    sectword = '【宗门信息】'
    w, h = await linewh(sectinfo, sectword)
    sectinfo_draw = ImageDraw.Draw(sectinfo)
    sectinfo_draw.text((w, h), sectword, first_color, font_40, 'lm')
    img.paste(sectinfo, (100, 1050), sectinfo)
    
    DETAIL_sectinfo = {}
    DETAIL_sectinfo['所在宗门'] = DETAIL_MAP['所在宗门']
    DETAIL_sectinfo['宗门职位'] = DETAIL_MAP['宗门职位']
    tasks3 = []
    for key, value in DETAIL_sectinfo.items():
        tasks3.append(_draw_sect_info_line(img, key, value, DETAIL_sectinfo))
    await asyncio.gather(*tasks3)
    
    res = await convert_img(img)
    return res

async def _draw_line(img: Image.Image, key, value, DETAIL_MAP):

    line = Image.open(TEXT_PATH / 'line3.png').resize((450, 68))
    line_draw = ImageDraw.Draw(line)
    word = f"{key}：{value}"
    w, h = await linewh(line, word)
    
    line_draw.text((70, h), word, first_color, font_36, 'lm')
    img.paste(line, (550, 100 + list(DETAIL_MAP.keys()).index(key) * 103), line)
    
async def _draw_base_info_line(img: Image.Image, key, value, DETAIL_MAP):

    line = Image.open(TEXT_PATH / 'line4.png').resize((900, 100))
    line_draw = ImageDraw.Draw(line)
    word = f"{key}：{value}"
    w, h = await linewh(line, word)
    
    line_draw.text((100, h), word, first_color, font_36, 'lm')
    img.paste(line, (100, 703 + list(DETAIL_MAP.keys()).index(key) * 103), line)
    
async def _draw_sect_info_line(img: Image.Image, key, value, DETAIL_MAP):

    line = Image.open(TEXT_PATH / 'line4.png').resize((900, 100))
    line_draw = ImageDraw.Draw(line)
    word = f"{key}：{value}"
    w, h = await linewh(line, word)
    
    line_draw.text((100, h), word, first_color, font_36, 'lm')
    img.paste(line, (100, 1153 + list(DETAIL_MAP.keys()).index(key) * 103), line)
    
   

async def img_author(img, bg):
    
    w, h = img.size
    alpha_layer = Image.new('L', (w, w), 0)
    draw = ImageDraw.Draw(alpha_layer)
    draw.ellipse((0, 0, w, w), fill=255)
    bg.paste(img, (88, 80), alpha_layer)
    
    return bg

async def linewh(line, word):
    lw, lh = line.size
    gs_font_36 = font_origin(36)
    w, h = gs_font_36.getsize(word)
    return (lw - w) / 2, lh / 2